#include <ninja/spinors.hh>
#include <ninja/ninjanumgen.hh>
#include "ttbarh_num.hh"
using namespace ninja;

Real TTbarHDiagram::mH = 125.0;
Real TTbarHDiagram::mT = 171.2;

void TTbarHDiagram::init(const ninja::RealMomentum k[])
{
  const ninja::Real mHq = mH*mH;
  const ninja::Real mTq = mT*mT;

  k1 = k[0];
  k2 = k[1];
  k3 = k[2];
  k4 = k[3];
  k5 = k[4];
  l3 = k3 - Real(0.5)*mHq/mp(k3,k2) * k2;
  l4 = k4 - Real(0.5)*mTq/mp(k4,k2) * k2;
  l5 = k5 - Real(0.5)*mTq/mp(k5,k2) * k2;

  es45 = mTq
    + Real(2)*mp(-k4, -k5)
    + mTq;
  es12 = Real(2)*mp(k1, k2);
  es3 = mHq;
  es51 = mTq
    + Real(2)*mp(-k5, k1);
  es23 = Real(2)*mp(k2, -k3)
    + mHq;
  es5 = mTq;
  es4 = mTq;
  es34 = mHq
    + Real(2)*mp(-k3, -k4)
    + mTq;

  Spinor spk1 = Spinor(k1);
  Spinor spk2 = Spinor(k2);
  //Spinor spk3 = Spinor(k3);
  //Spinor spk4 = Spinor(k4);
  //Spinor spk5 = Spinor(k5);
  Spinor spl3 = Spinor(l3);
  Spinor spl4 = Spinor(l4);
  Spinor spl5 = Spinor(l5);

  spak1k2 = spaa(spk1, spk2);
  spbk2k1 = spbb(spk2, spk1);
  spak1l3 = spaa(spk1, spl3);
  spbl3k1 = spbb(spl3, spk1);
  spak1l4 = spaa(spk1, spl4);
  spbl4k1 = spbb(spl4, spk1);
  spak1l5 = spaa(spk1, spl5);
  spbl5k1 = spbb(spl5, spk1);
  spak2l3 = spaa(spk2, spl3);
  spbl3k2 = spbb(spl3, spk2);
  spak2l4 = spaa(spk2, spl4);
  spbl4k2 = spbb(spl4, spk2);
  spak2l5 = spaa(spk2, spl5);
  spbl5k2 = spbb(spl5, spk2);
  spal3l4 = spaa(spl3, spl4);
  spbl4l3 = spbb(spl4, spl3);
  spal3l5 = spaa(spl3, spl5);
  spbl5l3 = spbb(spl5, spl3);
  spal4l5 = spaa(spl4, spl5);
  spbl5l4 = spbb(spl5, spl4);
  spvak1k2 = Real(2)*momentumFromSpinors(spk1, spk2);
  spvak1l3 = Real(2)*momentumFromSpinors(spk1, spl3);
  spvak1l4 = Real(2)*momentumFromSpinors(spk1, spl4);
  spvak1l5 = Real(2)*momentumFromSpinors(spk1, spl5);
  spvak2k1 = Real(2)*momentumFromSpinors(spk2, spk1);
  spvak2l3 = Real(2)*momentumFromSpinors(spk2, spl3);
  spvak2l4 = Real(2)*momentumFromSpinors(spk2, spl4);
  spvak2l5 = Real(2)*momentumFromSpinors(spk2, spl5);
  spval3k1 = Real(2)*momentumFromSpinors(spl3, spk1);
  spval3k2 = Real(2)*momentumFromSpinors(spl3, spk2);
  spval3l4 = Real(2)*momentumFromSpinors(spl3, spl4);
  spval3l5 = Real(2)*momentumFromSpinors(spl3, spl5);
  spval4k1 = Real(2)*momentumFromSpinors(spl4, spk1);
  spval4k2 = Real(2)*momentumFromSpinors(spl4, spk2);
  spval4l3 = Real(2)*momentumFromSpinors(spl4, spl3);
  spval4l5 = Real(2)*momentumFromSpinors(spl4, spl5);
  spval5k1 = Real(2)*momentumFromSpinors(spl5, spk1);
  spval5k2 = Real(2)*momentumFromSpinors(spl5, spk2);
  spval5l3 = Real(2)*momentumFromSpinors(spl5, spl3);
  spval5l4 = Real(2)*momentumFromSpinors(spl5, spl4);

  // pol. vec.s all left handed (but outgoing are reversed)
  e1 = polarizationVectorL(k2,k1);
  e2 = polarizationVectorL(k1,k2);

  Spinor spe1 = Spinor(e1);
  Spinor spe2 = Spinor(e2);

  spak1e1 = spaa(spk1,spe1);
  spbe1k1 = spbb(spe1,spk1);
  spae1k1 = spaa(spe1,spk1);
  spbk1e1 = spbb(spk1,spe1);
  spak1e2 = spaa(spk1,spe2);
  spbe2k1 = spbb(spe2,spk1);
  spae1k2 = spaa(spe1,spk2);
  spbk2e1 = spbb(spk2,spe1);
  spae1l3 = spaa(spe1,spl3);
  spbl3e1 = spbb(spl3,spe1);
  spae1l4 = spaa(spe1,spl4);
  spbl4e1 = spbb(spl4,spe1);
  spae1l5 = spaa(spe1,spl5);
  spbl5e1 = spbb(spl5,spe1);
  spak2e2 = spaa(spk2,spe2);
  spbe2k2 = spbb(spe2,spk2);
  spae2k2 = spaa(spe2,spk2);
  spbk2e2 = spbb(spk2,spe2);
  spae2l3 = spaa(spe2,spl3);
  spbl3e2 = spbb(spl3,spe2);
  spae2l4 = spaa(spe2,spl4);
  spbl4e2 = spbb(spl4,spe2);
  spae2l5 = spaa(spe2,spl5);
  spbl5e2 = spbb(spl5,spe2);
  spae1e2 = spaa(spe1,spe2);
  spbe2e1 = spbb(spe2,spe1);
  spvak1e1 = Real(2)*momentumFromSpinors(spk1,spe1);
  spvae1k1 = Real(2)*momentumFromSpinors(spe1,spk1);
  spvak1e2 = Real(2)*momentumFromSpinors(spk1,spe2);
  spvae2k1 = Real(2)*momentumFromSpinors(spe2,spk1);
  spvak2e1 = Real(2)*momentumFromSpinors(spk2,spe1);
  spvae1k2 = Real(2)*momentumFromSpinors(spe1,spk2);
  spvak2e2 = Real(2)*momentumFromSpinors(spk2,spe2);
  spvae2k2 = Real(2)*momentumFromSpinors(spe2,spk2);
  spval3e1 = Real(2)*momentumFromSpinors(spl3,spe1);
  spvae1l3 = Real(2)*momentumFromSpinors(spe1,spl3);
  spval3e2 = Real(2)*momentumFromSpinors(spl3,spe2);
  spvae2l3 = Real(2)*momentumFromSpinors(spe2,spl3);
  spval4e1 = Real(2)*momentumFromSpinors(spl4,spe1);
  spvae1l4 = Real(2)*momentumFromSpinors(spe1,spl4);
  spval4e2 = Real(2)*momentumFromSpinors(spl4,spe2);
  spvae2l4 = Real(2)*momentumFromSpinors(spe2,spl4);
  spval5e1 = Real(2)*momentumFromSpinors(spl5,spe1);
  spvae1l5 = Real(2)*momentumFromSpinors(spe1,spl5);
  spval5e2 = Real(2)*momentumFromSpinors(spl5,spe2);
  spvae2l5 = Real(2)*momentumFromSpinors(spe2,spl5);
  spvae1e2 = Real(2)*momentumFromSpinors(spe1,spe2);
  spvae2e1 = Real(2)*momentumFromSpinors(spe2,spe1);

  // shift (defined by GoSam)
  RealMomentum shift = k2;

  // internal momenta and masses
  msq[1-1] = Real(0);
  pi[1-1] = -k3-k5-k4+shift;
  msq[2-1] = TTbarHDiagram::mT*TTbarHDiagram::mT;
  pi[2-1] = -k3-k4+shift;
  msq[3-1] = TTbarHDiagram::mT*TTbarHDiagram::mT;
  pi[3-1] = -k4+shift;
  msq[4-1] = Real(0);
  pi[4-1] = shift;
  msq[5-1] = Real(0);
  pi[5-1] = -k2+shift;


  // S-matrix
  if (s_mat.isNull())
    s_mat.allocate(5);
  s_mat.fillFromKinematics(pi,1.0e-8);

  init_global_abbreviations_();

}


void TTbarHDiagram::init_global_abbreviations_()
{
  // ignore overall constants and color factors
  const Real gHT=1;
  const Real gs=1;
  const Real NC=1;
  const Real TR=1;
  const Real e=1;
  const Real c2 = 1;
  const Real c3 = 1;

  if (!globabbr.allocated())
    globabbr.allocate(158);

  // diagram 266, helicity4

  globabbr ( 1 ) = (NINJA_REAL(1)/powi1(spbl5k2 )); 
  globabbr ( 2 ) = (NINJA_REAL(1)/powi1(spak2l4 )); 
  globabbr ( 3 ) = (NINJA_REAL(1)/powi1(spbl4k2 )); 
  globabbr ( 4 ) = (NINJA_REAL(1)/powi1(spak2l5 )); 
  globabbr ( 5 ) = mT ; 
  globabbr ( 6 ) = NC * c2 ; 
  globabbr ( 6 ) = globabbr ( 6 ) + c3 ; 
  globabbr ( 7 ) = I * TR * e * gHT ; 
  globabbr ( 8 ) = powi4(gs ); 
  globabbr ( 9 ) = globabbr ( 6 ) * globabbr ( 7 ) * globabbr ( 8 ) * mT ; 
  globabbr ( 10 ) = spae2k2 * globabbr ( 2 ) ; 
  globabbr ( 11 ) = globabbr ( 10 ) * spae1l5 ; 
  globabbr ( 12 ) = globabbr ( 11 ) * globabbr ( 9 ) ; 
  globabbr ( 13 ) = globabbr ( 12 ) * spbl4e2 ; 
  globabbr ( 14 ) = spbk1e1 * globabbr ( 13 ) * spak1l4 ; 
  globabbr ( 15 ) = spal4l5 * spbl5e1 ; 
  globabbr ( 13 ) = globabbr ( 15 ) * globabbr ( 13 ) ; 
  globabbr ( 6 ) = globabbr ( 8 ) * globabbr ( 6 ) ; 
  globabbr ( 8 ) = globabbr ( 6 ) * globabbr ( 7 ) * globabbr ( 1 ) ; 
  globabbr ( 16 ) = powi2(mT ); 
  globabbr ( 17 ) = globabbr ( 16 ) * globabbr ( 8 ) ; 
  globabbr ( 18 ) = globabbr ( 17 ) * globabbr ( 5 ) ; 
  globabbr ( 19 ) = spbk2e1 * spbl5e2 * globabbr ( 18 ) * globabbr ( 11 ) ; 
  globabbr ( 20 ) = globabbr ( 18 ) * globabbr ( 10 ) ; 
  globabbr ( 21 ) = spbk2e1 * spbl4e2 ; 
  globabbr ( 22 ) = spae1l4 * globabbr ( 20 ) * globabbr ( 21 ) ; 
  globabbr ( 6 ) = globabbr ( 7 ) * globabbr ( 6 ) ; 
  globabbr ( 7 ) = globabbr ( 6 ) * globabbr ( 5 ) ; 
  globabbr ( 23 ) = globabbr ( 7 ) * spbl4e2 ; 
  globabbr ( 24 ) = spak1e2 * spbk1e1 ; 
  globabbr ( 25 ) = globabbr ( 24 ) * globabbr ( 23 ) * spae1l5 ; 
  globabbr ( 26 ) = globabbr ( 23 ) * spae2l5 ; 
  globabbr ( 27 ) = spbl5e1 * spae1l5 ; 
  globabbr ( 28 ) = globabbr ( 27 ) * globabbr ( 26 ) ; 
  globabbr ( 29 ) = globabbr ( 20 ) * spbe2k1 ; 
  globabbr ( 30 ) = spae1k1 * spbk2e1 ; 
  globabbr ( 31 ) = globabbr ( 29 ) * globabbr ( 30 ) ; 
  globabbr ( 13 ) = - globabbr ( 14 ) + globabbr ( 19 ) - globabbr ( 22 ) - globabbr ( 25 ) - 
    globabbr ( 28 ) + globabbr ( 31 ) - globabbr ( 13 ) ; 
  globabbr ( 14 ) = globabbr ( 1 ) * globabbr ( 9 ) ; 
  globabbr ( 19 ) = globabbr ( 14 ) * spbl4e2 ; 
  globabbr ( 22 ) = globabbr ( 5 ) * globabbr ( 5 ); 
  globabbr ( 25 ) = spae1e2 * globabbr ( 22 ) * globabbr ( 19 ) ; 
  globabbr ( 28 ) = globabbr ( 7 ) - globabbr ( 9 ) ; 
  globabbr ( 31 ) = spae2k2 * spbl4e2 ; 
  globabbr ( 32 ) = globabbr ( 28 ) * globabbr ( 31 ) ; 
  globabbr ( 33 ) = globabbr ( 32 ) * spae1l5 ; 
  globabbr ( 25 ) = globabbr ( 25 ) + globabbr ( 33 ) ; 
  globabbr ( 25 ) = globabbr ( 25 ) * spbk2e1 ; 
  globabbr ( 33 ) = globabbr ( 12 ) * spbe2e1 ; 
  globabbr ( 34 ) = globabbr ( 33 ) * globabbr ( 22 ) ; 
  globabbr ( 35 ) = powi3(mT ); 
  globabbr ( 36 ) = globabbr ( 35 ) * globabbr ( 8 ) ; 
  globabbr ( 37 ) = globabbr ( 18 ) - globabbr ( 36 ) ; 
  globabbr ( 37 ) = globabbr ( 37 ) * globabbr ( 4 ) ; 
  globabbr ( 38 ) = globabbr ( 31 ) * globabbr ( 37 ) ; 
  globabbr ( 39 ) = spbk2e1 * spae1l5 ; 
  globabbr ( 40 ) = globabbr ( 38 ) * globabbr ( 39 ) ; 
  globabbr ( 25 ) = globabbr ( 25 ) + globabbr ( 34 ) + globabbr ( 40 ) ; 
  globabbr ( 34 ) = globabbr ( 14 ) * spbl5l4 ; 
  globabbr ( 40 ) = spae1l5 * globabbr ( 34 ) ; 
  globabbr ( 41 ) = globabbr ( 14 ) * spbl4k1 ; 
  globabbr ( 42 ) = spae1k1 * globabbr ( 41 ) ; 
  globabbr ( 40 ) = globabbr ( 40 ) + globabbr ( 42 ) ; 
  globabbr ( 40 ) = globabbr ( 21 ) * globabbr ( 40 ) ; 
  globabbr ( 42 ) = spbl4e1 * spae1l5 ; 
  globabbr ( 43 ) = globabbr ( 42 ) * globabbr ( 23 ) ; 
  globabbr ( 40 ) = - globabbr ( 43 ) + globabbr ( 40 ) ; 
  globabbr ( 40 ) = globabbr ( 40 ) * spae2l4 ; 
  globabbr ( 43 ) = globabbr ( 14 ) * spbk2e1 ; 
  globabbr ( 44 ) = globabbr ( 43 ) * globabbr ( 31 ) ; 
  globabbr ( 45 ) = globabbr ( 36 ) * globabbr ( 10 ) ; 
  globabbr ( 46 ) = globabbr ( 3 ) * globabbr ( 21 ) * globabbr ( 45 ) ; 
  globabbr ( 44 ) = globabbr ( 44 ) - globabbr ( 46 ) ; 
  globabbr ( 46 ) = globabbr ( 44 ) * spae1k1 * spbk2k1 ; 
  globabbr ( 40 ) = globabbr ( 40 ) - globabbr ( 46 ) ; 
  globabbr ( 16 ) = globabbr ( 16 ) * globabbr ( 6 ) ; 
  globabbr ( 46 ) = globabbr ( 5 ) * globabbr ( 2 ) ; 
  globabbr ( 47 ) = globabbr ( 16 ) * globabbr ( 46 ) ; 
  globabbr ( 48 ) = globabbr ( 35 ) * globabbr ( 6 ) ; 
  globabbr ( 49 ) = globabbr ( 48 ) * globabbr ( 2 ) ; 
  globabbr ( 50 ) = globabbr ( 47 ) - globabbr ( 49 ) ; 
  globabbr ( 51 ) = globabbr ( 50 ) * globabbr ( 3 ) ; 
  globabbr ( 52 ) = globabbr ( 39 ) * globabbr ( 31 ) ; 
  globabbr ( 53 ) = globabbr ( 51 ) * globabbr ( 52 ) ; 
  globabbr ( 54 ) = globabbr ( 14 ) * spbl4k2 ; 
  globabbr ( 55 ) = spae1e2 * spbe2e1 ; 
  globabbr ( 56 ) = globabbr ( 54 ) * globabbr ( 55 ) ; 
  globabbr ( 57 ) = globabbr ( 22 ) * globabbr ( 56 ) ; 
  globabbr ( 57 ) = globabbr ( 40 ) + NINJA_REAL(1)/ NINJA_REAL(4)* globabbr ( 57 ) + globabbr ( 25 ) - globabbr ( 13 ) - 
    globabbr ( 53 ) ; 
  globabbr ( 57 ) = es12 * globabbr ( 57 ) ; 
  globabbr ( 58 ) = spbk2k1 * spak1l5 ; 
  globabbr ( 59 ) = globabbr ( 58 ) * spbl5l4 ; 
  globabbr ( 60 ) = globabbr ( 56 ) * globabbr ( 59 ) ; 
  globabbr ( 6 ) = globabbr ( 6 ) * globabbr ( 1 ) * globabbr ( 1 ); 
  globabbr ( 35 ) = globabbr ( 35 ) * globabbr ( 6 ) ; 
  globabbr ( 61 ) = globabbr ( 35 ) * globabbr ( 4 ) ; 
  globabbr ( 62 ) = powi2(spbl4k2 ); 
  globabbr ( 63 ) = globabbr ( 62 ) * globabbr ( 61 ) ; 
  globabbr ( 64 ) = - es12 * globabbr ( 55 ) * globabbr ( 63 ) ; 
  globabbr ( 65 ) = globabbr ( 55 ) * spbl4k2 ; 
  globabbr ( 66 ) = globabbr ( 65 ) * globabbr ( 7 ) ; 
  globabbr ( 67 ) = spak1l5 * spbl4k1 ; 
  globabbr ( 68 ) = - globabbr ( 67 ) * globabbr ( 66 ) ; 
  globabbr ( 60 ) = globabbr ( 60 ) + globabbr ( 64 ) + globabbr ( 68 ) ; 
  globabbr ( 64 ) = NINJA_REAL(1)/ NINJA_REAL(4)* spak2l4 ; 
  globabbr ( 60 ) = globabbr ( 60 ) * globabbr ( 64 ) ; 
  globabbr ( 68 ) = NINJA_REAL(1)/ NINJA_REAL(4)* spak1l5 ; 
  globabbr ( 69 ) = spbl5k1 * spak2l5 ; 
  globabbr ( 68 ) = globabbr ( 66 ) * globabbr ( 69 ) * globabbr ( 68 ) ; 
  globabbr ( 57 ) = globabbr ( 68 ) + globabbr ( 60 ) + globabbr ( 57 ) ; 
  globabbr ( 60 ) = - globabbr ( 51 ) + globabbr ( 37 ) + globabbr ( 28 ) ; 
  globabbr ( 68 ) = NINJA_REAL(2)* spak1k2 ; 
  globabbr ( 70 ) = - globabbr ( 68 ) * globabbr ( 39 ) * globabbr ( 60 ) ; 
  globabbr ( 71 ) = globabbr ( 9 ) * globabbr ( 2 ) ; 
  globabbr ( 72 ) = globabbr ( 71 ) * spae1l5 ; 
  globabbr ( 73 ) = globabbr ( 68 ) * globabbr ( 72 ) ; 
  globabbr ( 15 ) = - globabbr ( 15 ) * globabbr ( 73 ) ; 
  globabbr ( 15 ) = globabbr ( 15 ) + globabbr ( 70 ) ; 
  globabbr ( 15 ) = spbl4k1 * globabbr ( 15 ) ; 
  globabbr ( 60 ) = globabbr ( 60 ) * spbl4e1 ; 
  globabbr ( 70 ) = spae1k2 * spbk2k1 ; 
  globabbr ( 74 ) = globabbr ( 70 ) * spak1l5 * globabbr ( 60 ) ; 
  globabbr ( 75 ) = globabbr ( 43 ) * spbl4k1 ; 
  globabbr ( 76 ) = - globabbr ( 22 ) + es12 ; 
  globabbr ( 76 ) = globabbr ( 76 ) * globabbr ( 75 ) ; 
  globabbr ( 77 ) = globabbr ( 36 ) * globabbr ( 2 ) ; 
  globabbr ( 78 ) = globabbr ( 77 ) * globabbr ( 3 ) ; 
  globabbr ( 79 ) = spbl4k1 * spbk2e1 ; 
  globabbr ( 80 ) = globabbr ( 79 ) * globabbr ( 78 ) ; 
  globabbr ( 81 ) = - es12 * globabbr ( 80 ) ; 
  globabbr ( 76 ) = globabbr ( 76 ) + globabbr ( 81 ) ; 
  globabbr ( 81 ) = spbk2k1 * spbl4e1 ; 
  globabbr ( 82 ) = globabbr ( 81 ) * globabbr ( 14 ) ; 
  globabbr ( 83 ) = - globabbr ( 22 ) * globabbr ( 82 ) ; 
  globabbr ( 76 ) = NINJA_REAL(2)* globabbr ( 76 ) + globabbr ( 83 ) ; 
  globabbr ( 76 ) = spae1k1 * globabbr ( 76 ) ; 
  globabbr ( 48 ) = globabbr ( 48 ) * globabbr ( 3 ) ; 
  globabbr ( 83 ) = globabbr ( 2 ) * globabbr ( 2 ); 
  globabbr ( 84 ) = globabbr ( 48 ) * globabbr ( 83 ) ; 
  globabbr ( 85 ) = - spae1l5 * globabbr ( 84 ) ; 
  globabbr ( 85 ) = globabbr ( 85 ) + globabbr ( 72 ) ; 
  globabbr ( 86 ) = NINJA_REAL(2)* es12 ; 
  globabbr ( 85 ) = globabbr ( 86 ) * globabbr ( 85 ) ; 
  globabbr ( 87 ) = globabbr ( 22 ) * globabbr ( 72 ) ; 
  globabbr ( 88 ) = globabbr ( 7 ) * spbl4k2 ; 
  globabbr ( 89 ) = globabbr ( 88 ) * spae1l5 ; 
  globabbr ( 89 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 89 ) ; 
  globabbr ( 85 ) = - NINJA_REAL(2)* globabbr ( 87 ) + globabbr ( 89 ) + globabbr ( 85 ) ; 
  globabbr ( 85 ) = spak1k2 * globabbr ( 85 ) ; 
  globabbr ( 87 ) = globabbr ( 71 ) * spae1k2 ; 
  globabbr ( 90 ) = globabbr ( 87 ) * spak1l5 ; 
  globabbr ( 91 ) = - globabbr ( 22 ) * globabbr ( 90 ) ; 
  globabbr ( 85 ) = globabbr ( 85 ) + globabbr ( 91 ) ; 
  globabbr ( 85 ) = spbk1e1 * globabbr ( 85 ) ; 
  globabbr ( 91 ) = globabbr ( 67 ) * globabbr ( 7 ) ; 
  globabbr ( 92 ) = globabbr ( 84 ) * spak2l5 ; 
  globabbr ( 93 ) = - globabbr ( 86 ) * globabbr ( 92 ) ; 
  globabbr ( 93 ) = globabbr ( 93 ) - NINJA_REAL(2)* globabbr ( 91 ) ; 
  globabbr ( 93 ) = spae1l5 * globabbr ( 93 ) ; 
  globabbr ( 94 ) = globabbr ( 72 ) * globabbr ( 86 ) ; 
  globabbr ( 89 ) = globabbr ( 89 ) + globabbr ( 94 ) ; 
  globabbr ( 89 ) = spak2l5 * globabbr ( 89 ) ; 
  globabbr ( 94 ) = globabbr ( 17 ) * globabbr ( 46 ) ; 
  globabbr ( 95 ) = globabbr ( 94 ) * spae1k2 ; 
  globabbr ( 96 ) = - globabbr ( 58 ) * globabbr ( 95 ) ; 
  globabbr ( 89 ) = globabbr ( 96 ) + globabbr ( 89 ) + globabbr ( 93 ) ; 
  globabbr ( 89 ) = spbl5e1 * globabbr ( 89 ) ; 
  globabbr ( 93 ) = globabbr ( 34 ) * globabbr ( 58 ) ; 
  globabbr ( 96 ) = globabbr ( 61 ) * spbl4k2 ; 
  globabbr ( 97 ) = - es12 * globabbr ( 96 ) ; 
  globabbr ( 93 ) = globabbr ( 93 ) + globabbr ( 97 ) - globabbr ( 91 ) ; 
  globabbr ( 93 ) = spbl4e1 * globabbr ( 93 ) ; 
  globabbr ( 97 ) = globabbr ( 68 ) * globabbr ( 94 ) ; 
  globabbr ( 98 ) = - globabbr ( 79 ) * globabbr ( 97 ) ; 
  globabbr ( 93 ) = globabbr ( 98 ) + globabbr ( 93 ) ; 
  globabbr ( 93 ) = spae1l4 * globabbr ( 93 ) ; 
  globabbr ( 98 ) = spal4l5 * spbl4e1 ; 
  globabbr ( 99 ) = globabbr ( 90 ) * globabbr ( 98 ) ; 
  globabbr ( 100 ) = globabbr ( 42 ) * globabbr ( 7 ) ; 
  globabbr ( 101 ) = spak1l5 * globabbr ( 100 ) ; 
  globabbr ( 97 ) = globabbr ( 39 ) * globabbr ( 97 ) ; 
  globabbr ( 97 ) = globabbr ( 99 ) + globabbr ( 101 ) + globabbr ( 97 ) ; 
  globabbr ( 97 ) = spbl5k1 * globabbr ( 97 ) ; 
  globabbr ( 99 ) = NINJA_REAL(2)* globabbr ( 14 ) ; 
  globabbr ( 101 ) = globabbr ( 39 ) * spbl5l4 ; 
  globabbr ( 102 ) = - globabbr ( 99 ) * globabbr ( 101 ) ; 
  globabbr ( 103 ) = - spbk1e1 * globabbr ( 73 ) ; 
  globabbr ( 102 ) = globabbr ( 103 ) + globabbr ( 102 ) + NINJA_REAL(2)* globabbr ( 100 ) ; 
  globabbr ( 102 ) = spbl4k1 * globabbr ( 102 ) ; 
  globabbr ( 103 ) = - powi2(spbl4k1 )* globabbr ( 99 ) * globabbr ( 30 ) ; 
  globabbr ( 104 ) = globabbr ( 95 ) * globabbr ( 81 ) ; 
  globabbr ( 102 ) = globabbr ( 103 ) + globabbr ( 104 ) + globabbr ( 102 ) ; 
  globabbr ( 102 ) = spak1l4 * globabbr ( 102 ) ; 
  globabbr ( 30 ) = spbl4k1 * globabbr ( 30 ) ; 
  globabbr ( 30 ) = globabbr ( 101 ) + globabbr ( 30 ) ; 
  globabbr ( 30 ) = globabbr ( 54 ) * globabbr ( 30 ) ; 
  globabbr ( 101 ) = - globabbr ( 42 ) * globabbr ( 88 ) ; 
  globabbr ( 30 ) = globabbr ( 101 ) + globabbr ( 30 ) ; 
  globabbr ( 101 ) = NINJA_REAL(1)/ NINJA_REAL(2)* spak2l4 ; 
  globabbr ( 30 ) = globabbr ( 30 ) * globabbr ( 101 ) ; 
  globabbr ( 15 ) = globabbr ( 102 ) + globabbr ( 97 ) + globabbr ( 30 ) + globabbr ( 93 ) + globabbr ( 89 ) 
    + globabbr ( 85 ) + globabbr ( 76 ) + globabbr ( 74 ) + globabbr ( 15 ) ; 
  globabbr ( 30 ) = globabbr ( 31 ) * globabbr ( 51 ) ; 
  globabbr ( 30 ) = globabbr ( 30 ) - globabbr ( 32 ) - globabbr ( 38 ) ; 
  globabbr ( 30 ) = spak1l5 * globabbr ( 30 ) ; 
  globabbr ( 32 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 19 ) ; 
  globabbr ( 74 ) = - globabbr ( 22 ) * globabbr ( 32 ) ; 
  globabbr ( 76 ) = globabbr ( 19 ) * globabbr ( 86 ) ; 
  globabbr ( 85 ) = NINJA_REAL(2)* spak2l5 ; 
  globabbr ( 89 ) = - globabbr ( 23 ) * globabbr ( 85 ) ; 
  globabbr ( 74 ) = globabbr ( 89 ) + globabbr ( 74 ) + globabbr ( 76 ) ; 
  globabbr ( 74 ) = spak1e2 * globabbr ( 74 ) ; 
  globabbr ( 76 ) = NINJA_REAL(2)* globabbr ( 9 ) ; 
  globabbr ( 89 ) = - spak2l5 * globabbr ( 76 ) ; 
  globabbr ( 18 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 18 ) ; 
  globabbr ( 89 ) = - globabbr ( 18 ) + globabbr ( 89 ) ; 
  globabbr ( 93 ) = globabbr ( 10 ) * spbl4e2 ; 
  globabbr ( 89 ) = spak1l4 * globabbr ( 93 ) * globabbr ( 89 ) ; 
  globabbr ( 30 ) = globabbr ( 89 ) + globabbr ( 74 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 30 ) ; 
  globabbr ( 30 ) = spbk2k1 * globabbr ( 30 ) ; 
  globabbr ( 74 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 67 ) ; 
  globabbr ( 85 ) = - spbl4k2 * globabbr ( 85 ) ; 
  globabbr ( 85 ) = globabbr ( 74 ) + globabbr ( 85 ) ; 
  globabbr ( 85 ) = globabbr ( 23 ) * globabbr ( 85 ) ; 
  globabbr ( 89 ) = NINJA_REAL(2)* globabbr ( 54 ) ; 
  globabbr ( 97 ) = - globabbr ( 89 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 96 ) ; 
  globabbr ( 97 ) = es12 * spbl4e2 * globabbr ( 97 ) ; 
  globabbr ( 102 ) = - globabbr ( 32 ) * globabbr ( 59 ) ; 
  globabbr ( 85 ) = globabbr ( 102 ) + globabbr ( 97 ) + globabbr ( 85 ) ; 
  globabbr ( 85 ) = spae2l4 * globabbr ( 85 ) ; 
  globabbr ( 97 ) = globabbr ( 9 ) * globabbr ( 10 ) ; 
  globabbr ( 102 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 97 ) ; 
  globabbr ( 103 ) = - globabbr ( 22 ) * globabbr ( 102 ) ; 
  globabbr ( 104 ) = globabbr ( 76 ) * globabbr ( 10 ) ; 
  globabbr ( 105 ) = es12 * globabbr ( 104 ) ; 
  globabbr ( 103 ) = globabbr ( 103 ) + globabbr ( 105 ) ; 
  globabbr ( 103 ) = spak1l5 * globabbr ( 103 ) ; 
  globabbr ( 105 ) = globabbr ( 88 ) * spae2l5 ; 
  globabbr ( 106 ) = spak1k2 * globabbr ( 105 ) ; 
  globabbr ( 103 ) = globabbr ( 103 ) + globabbr ( 106 ) ; 
  globabbr ( 103 ) = spbe2k1 * globabbr ( 103 ) ; 
  globabbr ( 106 ) = spak1e2 * spbl4k1 ; 
  globabbr ( 107 ) = spbl5l4 * spae2l5 ; 
  globabbr ( 106 ) = globabbr ( 106 ) - globabbr ( 107 ) ; 
  globabbr ( 108 ) = spbk2e2 * globabbr ( 54 ) * globabbr ( 106 ) ; 
  globabbr ( 109 ) = spbl4k2 * globabbr ( 26 ) ; 
  globabbr ( 108 ) = globabbr ( 108 ) + globabbr ( 109 ) ; 
  globabbr ( 108 ) = spak2l4 * globabbr ( 108 ) ; 
  globabbr ( 109 ) = NINJA_REAL(2)* globabbr ( 7 ) - globabbr ( 9 ) ; 
  globabbr ( 110 ) = spae2l5 * spbl4e2 ; 
  globabbr ( 109 ) = globabbr ( 109 ) * globabbr ( 86 ) * globabbr ( 110 ) ; 
  globabbr ( 111 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 20 ) ; 
  globabbr ( 112 ) = globabbr ( 58 ) * globabbr ( 111 ) ; 
  globabbr ( 105 ) = - spak2l5 * globabbr ( 105 ) ; 
  globabbr ( 105 ) = globabbr ( 112 ) + globabbr ( 105 ) ; 
  globabbr ( 105 ) = spbl5e2 * globabbr ( 105 ) ; 
  globabbr ( 112 ) = globabbr ( 93 ) * spal4l5 ; 
  globabbr ( 113 ) = globabbr ( 112 ) * globabbr ( 76 ) ; 
  globabbr ( 114 ) = es12 * globabbr ( 113 ) ; 
  globabbr ( 115 ) = globabbr ( 9 ) * globabbr ( 112 ) ; 
  globabbr ( 116 ) = globabbr ( 115 ) + globabbr ( 26 ) ; 
  globabbr ( 117 ) = NINJA_REAL(1)/ NINJA_REAL(2)* spbl5k1 ; 
  globabbr ( 118 ) = - globabbr ( 117 ) * spak1l5 * globabbr ( 116 ) ; 
  globabbr ( 116 ) = spbl5k2 * spak2l5 * globabbr ( 116 ) ; 
  globabbr ( 30 ) = NINJA_REAL(2)* globabbr ( 116 ) + globabbr ( 118 ) + globabbr ( 114 ) + globabbr ( 105 ) + 
    globabbr ( 103 ) + globabbr ( 85 ) + globabbr ( 109 ) + globabbr ( 30 ) + globabbr ( 108 ) ; 
  globabbr ( 85 ) = globabbr ( 9 ) * globabbr ( 46 ) ; 
  globabbr ( 16 ) = globabbr ( 16 ) * globabbr ( 2 ) ; 
  globabbr ( 103 ) = - NINJA_REAL(2)* globabbr ( 16 ) + NINJA_REAL(5)* globabbr ( 85 ) ; 
  globabbr ( 103 ) = globabbr ( 5 ) * globabbr ( 103 ) ; 
  globabbr ( 103 ) = - globabbr ( 88 ) + globabbr ( 49 ) + globabbr ( 103 ) ; 
  globabbr ( 103 ) = spak2l5 * globabbr ( 103 ) ; 
  globabbr ( 105 ) = globabbr ( 71 ) * spak2l5 ; 
  globabbr ( 108 ) = globabbr ( 105 ) - globabbr ( 54 ) ; 
  globabbr ( 109 ) = NINJA_REAL(4)* spal4l5 ; 
  globabbr ( 109 ) = - spbl5l4 * globabbr ( 108 ) * globabbr ( 109 ) ; 
  globabbr ( 114 ) = globabbr ( 14 ) * globabbr ( 5 ) ; 
  globabbr ( 116 ) = NINJA_REAL(2)* globabbr ( 17 ) - NINJA_REAL(5)* globabbr ( 114 ) ; 
  globabbr ( 116 ) = globabbr ( 5 ) * globabbr ( 116 ) ; 
  globabbr ( 116 ) = globabbr ( 116 ) - globabbr ( 36 ) ; 
  globabbr ( 116 ) = spbl4k2 * globabbr ( 116 ) ; 
  globabbr ( 118 ) = NINJA_REAL(5)* globabbr ( 77 ) ; 
  globabbr ( 119 ) = - globabbr ( 118 ) + NINJA_REAL(9)* globabbr ( 94 ) ; 
  globabbr ( 119 ) = es12 * globabbr ( 119 ) ; 
  globabbr ( 120 ) = NINJA_REAL(5)* globabbr ( 9 ) - NINJA_REAL(9)* globabbr ( 7 ) ; 
  globabbr ( 120 ) = globabbr ( 120 ) * globabbr ( 67 ) ; 
  globabbr ( 121 ) = globabbr ( 62 ) * globabbr ( 14 ) ; 
  globabbr ( 122 ) = globabbr ( 121 ) + NINJA_REAL(4)* globabbr ( 63 ) ; 
  globabbr ( 122 ) = spak2l4 * globabbr ( 122 ) ; 
  globabbr ( 123 ) = powi2(spak2l5 )* globabbr ( 71 ) ; 
  globabbr ( 124 ) = spak2l5 * globabbr ( 2 ) ; 
  globabbr ( 48 ) = - (globabbr ( 124 ) * globabbr ( 124 ))* globabbr ( 48 ) ; 
  globabbr ( 48 ) = globabbr ( 123 ) + globabbr ( 48 ) ; 
  globabbr ( 48 ) = spbl5k2 * globabbr ( 48 ) ; 
  globabbr ( 48 ) = NINJA_REAL(4)* globabbr ( 48 ) + globabbr ( 122 ) + globabbr ( 109 ) + globabbr ( 120 ) + 
    globabbr ( 103 ) + globabbr ( 116 ) + globabbr ( 119 ) ; 
  globabbr ( 103 ) = globabbr ( 94 ) - globabbr ( 77 ) ; 
  globabbr ( 109 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 55 ) ; 
  globabbr ( 116 ) = globabbr ( 103 ) * globabbr ( 109 ) ; 
  globabbr ( 116 ) = globabbr ( 116 ) + globabbr ( 56 ) ; 
  globabbr ( 21 ) = globabbr ( 21 ) * spae1e2 ; 
  globabbr ( 119 ) = globabbr ( 21 ) * globabbr ( 99 ) ; 
  globabbr ( 11 ) = globabbr ( 11 ) * spbe2e1 ; 
  globabbr ( 120 ) = globabbr ( 11 ) * globabbr ( 76 ) ; 
  globabbr ( 119 ) = globabbr ( 119 ) + globabbr ( 120 ) ; 
  globabbr ( 120 ) = globabbr ( 119 ) + globabbr ( 116 ) ; 
  globabbr ( 120 ) = es12 * globabbr ( 120 ) ; 
  globabbr ( 122 ) = globabbr ( 67 ) * globabbr ( 109 ) ; 
  globabbr ( 123 ) = - globabbr ( 28 ) * globabbr ( 122 ) ; 
  globabbr ( 124 ) = NINJA_REAL(1)/ NINJA_REAL(2)* spak2l5 ; 
  globabbr ( 125 ) = globabbr ( 66 ) * globabbr ( 124 ) ; 
  globabbr ( 120 ) = globabbr ( 123 ) + globabbr ( 120 ) + globabbr ( 125 ) ; 
  globabbr ( 79 ) = - globabbr ( 99 ) * globabbr ( 79 ) ; 
  globabbr ( 79 ) = globabbr ( 79 ) - globabbr ( 82 ) ; 
  globabbr ( 79 ) = spae1k1 * globabbr ( 79 ) ; 
  globabbr ( 82 ) = - globabbr ( 73 ) - globabbr ( 90 ) ; 
  globabbr ( 82 ) = spbk1e1 * globabbr ( 82 ) ; 
  globabbr ( 79 ) = globabbr ( 79 ) + globabbr ( 82 ) ; 
  globabbr ( 82 ) = NINJA_REAL(3)* globabbr ( 7 ) ; 
  globabbr ( 123 ) = globabbr ( 82 ) - globabbr ( 9 ) ; 
  globabbr ( 123 ) = globabbr ( 123 ) * globabbr ( 42 ) ; 
  globabbr ( 125 ) = NINJA_REAL(2)* globabbr ( 71 ) ; 
  globabbr ( 126 ) = globabbr ( 125 ) * spae1k2 ; 
  globabbr ( 127 ) = globabbr ( 98 ) * globabbr ( 126 ) ; 
  globabbr ( 123 ) = globabbr ( 123 ) + globabbr ( 127 ) ; 
  globabbr ( 79 ) = globabbr ( 123 ) + NINJA_REAL(2)* globabbr ( 79 ) ; 
  globabbr ( 127 ) = spak1e2 * spbk2k1 ; 
  globabbr ( 128 ) = globabbr ( 19 ) * globabbr ( 127 ) ; 
  globabbr ( 129 ) = - globabbr ( 76 ) + globabbr ( 82 ) ; 
  globabbr ( 129 ) = globabbr ( 129 ) * globabbr ( 110 ) ; 
  globabbr ( 10 ) = globabbr ( 10 ) * spbe2k1 ; 
  globabbr ( 130 ) = globabbr ( 10 ) * globabbr ( 9 ) ; 
  globabbr ( 131 ) = - spak1l5 * globabbr ( 130 ) ; 
  globabbr ( 129 ) = globabbr ( 115 ) - globabbr ( 128 ) + globabbr ( 129 ) + globabbr ( 131 ) ; 
  globabbr ( 131 ) = - globabbr ( 89 ) + globabbr ( 105 ) ; 
  globabbr ( 131 ) = NINJA_REAL(8)* globabbr ( 131 ) ; 
  globabbr ( 132 ) = globabbr ( 105 ) * globabbr ( 55 ) ; 
  globabbr ( 133 ) = globabbr ( 132 ) * spak1l5 ; 
  globabbr ( 134 ) = globabbr ( 133 ) * spbl5k1 ; 
  globabbr ( 135 ) = globabbr ( 61 ) * globabbr ( 65 ) ; 
  globabbr ( 136 ) = globabbr ( 94 ) * globabbr ( 55 ) ; 
  globabbr ( 137 ) = globabbr ( 135 ) - globabbr ( 136 ) ; 
  globabbr ( 138 ) = es12 * globabbr ( 137 ) ; 
  globabbr ( 139 ) = globabbr ( 14 ) * globabbr ( 55 ) ; 
  globabbr ( 140 ) = - globabbr ( 139 ) * globabbr ( 59 ) ; 
  globabbr ( 91 ) = globabbr ( 55 ) * globabbr ( 91 ) ; 
  globabbr ( 91 ) = - globabbr ( 134 ) + globabbr ( 140 ) + globabbr ( 138 ) + globabbr ( 91 ) ; 
  globabbr ( 91 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 91 ) ; 
  globabbr ( 138 ) = globabbr ( 72 ) * spak1k2 ; 
  globabbr ( 140 ) = globabbr ( 138 ) * spbk1e1 ; 
  globabbr ( 141 ) = globabbr ( 75 ) * spae1k1 ; 
  globabbr ( 140 ) = globabbr ( 140 ) + globabbr ( 141 ) ; 
  globabbr ( 27 ) = globabbr ( 27 ) * globabbr ( 105 ) ; 
  globabbr ( 141 ) = globabbr ( 43 ) * spae1l5 ; 
  globabbr ( 142 ) = spbl5l4 * globabbr ( 141 ) ; 
  globabbr ( 100 ) = globabbr ( 27 ) + globabbr ( 142 ) - globabbr ( 100 ) + globabbr ( 140 ) ; 
  globabbr ( 142 ) = globabbr ( 34 ) * spae2l5 ; 
  globabbr ( 143 ) = globabbr ( 41 ) * spak1e2 ; 
  globabbr ( 142 ) = globabbr ( 142 ) - globabbr ( 143 ) ; 
  globabbr ( 142 ) = globabbr ( 142 ) * spbk2e2 ; 
  globabbr ( 143 ) = spbl5e2 * spae2l5 ; 
  globabbr ( 144 ) = globabbr ( 143 ) * globabbr ( 105 ) ; 
  globabbr ( 142 ) = globabbr ( 142 ) + globabbr ( 144 ) ; 
  globabbr ( 144 ) = globabbr ( 71 ) * spak1k2 ; 
  globabbr ( 145 ) = globabbr ( 144 ) * spae2l5 ; 
  globabbr ( 146 ) = - spbe2k1 * globabbr ( 145 ) ; 
  globabbr ( 26 ) = - globabbr ( 26 ) + globabbr ( 146 ) + globabbr ( 142 ) ; 
  globabbr ( 26 ) = NINJA_REAL(2)* globabbr ( 26 ) ; 
  globabbr ( 146 ) = globabbr ( 54 ) + NINJA_REAL(3)* globabbr ( 105 ) ; 
  globabbr ( 146 ) = NINJA_REAL(2)* globabbr ( 146 ) ; 
  globabbr ( 13 ) = globabbr ( 13 ) - globabbr ( 40 ) ; 
  globabbr ( 40 ) = globabbr ( 25 ) - globabbr ( 13 ) ; 
  globabbr ( 50 ) = globabbr ( 50 ) * globabbr ( 52 ) ; 
  globabbr ( 52 ) = powi4(mT ); 
  globabbr ( 8 ) = globabbr ( 83 ) * globabbr ( 8 ) * globabbr ( 52 ) ; 
  globabbr ( 83 ) = globabbr ( 109 ) * globabbr ( 5 ) ; 
  globabbr ( 147 ) = - es12 * globabbr ( 8 ) * globabbr ( 83 ) ; 
  globabbr ( 148 ) = globabbr ( 47 ) * globabbr ( 122 ) ; 
  globabbr ( 50 ) = globabbr ( 148 ) + globabbr ( 50 ) + globabbr ( 147 ) ; 
  globabbr ( 50 ) = globabbr ( 3 ) * globabbr ( 50 ) ; 
  globabbr ( 147 ) = globabbr ( 21 ) * globabbr ( 35 ) ; 
  globabbr ( 11 ) = globabbr ( 11 ) * globabbr ( 36 ) ; 
  globabbr ( 11 ) = globabbr ( 147 ) + globabbr ( 11 ) ; 
  globabbr ( 147 ) = NINJA_REAL(2)* globabbr ( 11 ) ; 
  globabbr ( 6 ) = globabbr ( 46 ) * globabbr ( 52 ) * globabbr ( 6 ) ; 
  globabbr ( 52 ) = globabbr ( 109 ) * globabbr ( 6 ) ; 
  globabbr ( 35 ) = globabbr ( 35 ) * globabbr ( 65 ) ; 
  globabbr ( 148 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 35 ) ; 
  globabbr ( 52 ) = globabbr ( 148 ) + globabbr ( 52 ) + globabbr ( 147 ) ; 
  globabbr ( 52 ) = globabbr ( 4 ) * globabbr ( 52 ) ; 
  globabbr ( 149 ) = globabbr ( 109 ) * globabbr ( 94 ) ; 
  globabbr ( 52 ) = globabbr ( 149 ) + globabbr ( 52 ) ; 
  globabbr ( 52 ) = es12 * globabbr ( 52 ) ; 
  globabbr ( 149 ) = globabbr ( 109 ) * globabbr ( 54 ) ; 
  globabbr ( 22 ) = globabbr ( 22 ) * globabbr ( 149 ) ; 
  globabbr ( 150 ) = globabbr ( 37 ) + globabbr ( 7 ) ; 
  globabbr ( 74 ) = - globabbr ( 74 ) * globabbr ( 55 ) * globabbr ( 150 ) ; 
  globabbr ( 151 ) = globabbr ( 78 ) * globabbr ( 55 ) ; 
  globabbr ( 151 ) = globabbr ( 151 ) - globabbr ( 139 ) ; 
  globabbr ( 152 ) = NINJA_REAL(1)/ NINJA_REAL(2)* spbl5l4 ; 
  globabbr ( 153 ) = globabbr ( 151 ) * globabbr ( 152 ) ; 
  globabbr ( 58 ) = - globabbr ( 153 ) * globabbr ( 58 ) ; 
  globabbr ( 154 ) = globabbr ( 92 ) * globabbr ( 55 ) ; 
  globabbr ( 155 ) = - spak1l5 * globabbr ( 154 ) ; 
  globabbr ( 133 ) = globabbr ( 133 ) + globabbr ( 155 ) ; 
  globabbr ( 117 ) = globabbr ( 133 ) * globabbr ( 117 ) ; 
  globabbr ( 133 ) = globabbr ( 109 ) * spak2l4 ; 
  globabbr ( 63 ) = - globabbr ( 63 ) * globabbr ( 133 ) ; 
  globabbr ( 22 ) = globabbr ( 117 ) + globabbr ( 63 ) + globabbr ( 58 ) + globabbr ( 50 ) + globabbr ( 74 ) 
    + globabbr ( 52 ) + globabbr ( 22 ) - globabbr ( 40 ) ; 
  globabbr ( 50 ) = spbl4e1 * globabbr ( 150 ) ; 
  globabbr ( 52 ) = globabbr ( 92 ) - globabbr ( 105 ) ; 
  globabbr ( 58 ) = spbl5e1 * globabbr ( 52 ) ; 
  globabbr ( 50 ) = globabbr ( 58 ) + globabbr ( 50 ) ; 
  globabbr ( 50 ) = spae1l5 * globabbr ( 50 ) ; 
  globabbr ( 58 ) = globabbr ( 61 ) * spbk2e1 ; 
  globabbr ( 63 ) = - globabbr ( 43 ) - NINJA_REAL(4)* globabbr ( 58 ) ; 
  globabbr ( 63 ) = spbl4k1 * globabbr ( 63 ) ; 
  globabbr ( 74 ) = NINJA_REAL(2)* globabbr ( 61 ) ; 
  globabbr ( 117 ) = - globabbr ( 81 ) * globabbr ( 74 ) ; 
  globabbr ( 63 ) = globabbr ( 117 ) + globabbr ( 63 ) + globabbr ( 80 ) ; 
  globabbr ( 63 ) = spae1k1 * globabbr ( 63 ) ; 
  globabbr ( 117 ) = globabbr ( 77 ) * globabbr ( 4 ) ; 
  globabbr ( 155 ) = globabbr ( 84 ) - NINJA_REAL(4)* globabbr ( 117 ) ; 
  globabbr ( 155 ) = spae1l5 * globabbr ( 155 ) ; 
  globabbr ( 72 ) = - globabbr ( 72 ) + globabbr ( 155 ) ; 
  globabbr ( 72 ) = spak1k2 * globabbr ( 72 ) ; 
  globabbr ( 155 ) = NINJA_REAL(2)* globabbr ( 117 ) ; 
  globabbr ( 156 ) = - spae1k2 * spak1l5 * globabbr ( 155 ) ; 
  globabbr ( 72 ) = globabbr ( 72 ) + globabbr ( 156 ) ; 
  globabbr ( 72 ) = spbk1e1 * globabbr ( 72 ) ; 
  globabbr ( 39 ) = globabbr ( 39 ) * globabbr ( 78 ) ; 
  globabbr ( 39 ) = - globabbr ( 141 ) + globabbr ( 39 ) ; 
  globabbr ( 39 ) = spbl5l4 * globabbr ( 39 ) ; 
  globabbr ( 47 ) = globabbr ( 47 ) * globabbr ( 3 ) ; 
  globabbr ( 156 ) = - globabbr ( 42 ) * globabbr ( 47 ) ; 
  globabbr ( 157 ) = spae1l4 * spbl4e1 ; 
  globabbr ( 158 ) = globabbr ( 157 ) * globabbr ( 96 ) ; 
  globabbr ( 39 ) = - NINJA_REAL(2)* globabbr ( 158 ) + globabbr ( 72 ) + globabbr ( 39 ) + globabbr ( 63 ) + 
    globabbr ( 156 ) + globabbr ( 50 ) ; 
  globabbr ( 50 ) = - globabbr ( 47 ) + globabbr ( 37 ) + globabbr ( 82 ) + globabbr ( 9 ) ; 
  globabbr ( 50 ) = spae2l5 * globabbr ( 50 ) ; 
  globabbr ( 63 ) = globabbr ( 89 ) - globabbr ( 96 ) ; 
  globabbr ( 63 ) = spae2l4 * globabbr ( 63 ) ; 
  globabbr ( 50 ) = NINJA_REAL(2)* globabbr ( 50 ) + globabbr ( 63 ) ; 
  globabbr ( 50 ) = spbl4e2 * globabbr ( 50 ) ; 
  globabbr ( 63 ) = globabbr ( 78 ) - globabbr ( 14 ) ; 
  globabbr ( 72 ) = globabbr ( 63 ) * globabbr ( 107 ) ; 
  globabbr ( 107 ) = - spbl4k1 * globabbr ( 78 ) ; 
  globabbr ( 107 ) = globabbr ( 41 ) + globabbr ( 107 ) ; 
  globabbr ( 107 ) = spak1e2 * globabbr ( 107 ) ; 
  globabbr ( 72 ) = globabbr ( 72 ) + globabbr ( 107 ) ; 
  globabbr ( 72 ) = spbk2e2 * globabbr ( 72 ) ; 
  globabbr ( 52 ) = globabbr ( 52 ) * globabbr ( 143 ) ; 
  globabbr ( 52 ) = globabbr ( 52 ) + globabbr ( 72 ) ; 
  globabbr ( 45 ) = globabbr ( 4 ) * globabbr ( 45 ) ; 
  globabbr ( 45 ) = globabbr ( 45 ) - globabbr ( 104 ) ; 
  globabbr ( 72 ) = - spak1l5 * globabbr ( 45 ) ; 
  globabbr ( 107 ) = globabbr ( 84 ) - globabbr ( 71 ) ; 
  globabbr ( 143 ) = - spae2l5 * globabbr ( 107 ) * globabbr ( 68 ) ; 
  globabbr ( 72 ) = globabbr ( 72 ) + globabbr ( 143 ) ; 
  globabbr ( 72 ) = spbe2k1 * globabbr ( 72 ) ; 
  globabbr ( 143 ) = NINJA_REAL(2)* globabbr ( 19 ) ; 
  globabbr ( 156 ) = - spbl4e2 * globabbr ( 61 ) ; 
  globabbr ( 156 ) = - globabbr ( 143 ) + globabbr ( 156 ) ; 
  globabbr ( 127 ) = globabbr ( 156 ) * globabbr ( 127 ) ; 
  globabbr ( 50 ) = globabbr ( 113 ) + globabbr ( 127 ) + globabbr ( 72 ) + globabbr ( 50 ) + NINJA_REAL(2)* 
    globabbr ( 52 ) ; 
  globabbr ( 52 ) = - globabbr ( 92 ) - NINJA_REAL(4)* globabbr ( 96 ) + globabbr ( 118 ) - NINJA_REAL(8)* globabbr ( 94 ) + 
    globabbr ( 108 ) ; 
  globabbr ( 52 ) = NINJA_REAL(2)* globabbr ( 52 ) ; 
  globabbr ( 72 ) = globabbr ( 154 ) - globabbr ( 132 ) ; 
  globabbr ( 96 ) = globabbr ( 103 ) * globabbr ( 55 ) ; 
  globabbr ( 96 ) = NINJA_REAL(2)* globabbr ( 56 ) + globabbr ( 96 ) - globabbr ( 119 ) - globabbr ( 72 ) ; 
  globabbr ( 6 ) = globabbr ( 55 ) * globabbr ( 6 ) ; 
  globabbr ( 6 ) = globabbr ( 35 ) + globabbr ( 6 ) - globabbr ( 147 ) ; 
  globabbr ( 6 ) = globabbr ( 4 ) * globabbr ( 6 ) ; 
  globabbr ( 35 ) = globabbr ( 55 ) * globabbr ( 5 ) ; 
  globabbr ( 8 ) = - globabbr ( 3 ) * globabbr ( 8 ) * globabbr ( 35 ) ; 
  globabbr ( 6 ) = globabbr ( 8 ) + globabbr ( 136 ) + globabbr ( 6 ) ; 
  globabbr ( 8 ) = globabbr ( 21 ) * globabbr ( 14 ) ; 
  globabbr ( 8 ) = globabbr ( 8 ) + globabbr ( 33 ) ; 
  globabbr ( 21 ) = - globabbr ( 8 ) * globabbr ( 86 ) ; 
  globabbr ( 33 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 9 ) ; 
  globabbr ( 46 ) = globabbr ( 33 ) * globabbr ( 46 ) ; 
  globabbr ( 46 ) = globabbr ( 46 ) - globabbr ( 16 ) ; 
  globabbr ( 46 ) = globabbr ( 46 ) * globabbr ( 5 ) ; 
  globabbr ( 49 ) = globabbr ( 46 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 49 ) ; 
  globabbr ( 49 ) = globabbr ( 49 ) * globabbr ( 55 ) ; 
  globabbr ( 113 ) = - globabbr ( 9 ) + NINJA_REAL(3)/ NINJA_REAL(2)* globabbr ( 7 ) ; 
  globabbr ( 118 ) = globabbr ( 113 ) * globabbr ( 65 ) ; 
  globabbr ( 49 ) = globabbr ( 49 ) + globabbr ( 118 ) ; 
  globabbr ( 118 ) = spak2l5 * globabbr ( 49 ) ; 
  globabbr ( 127 ) = - globabbr ( 17 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 114 ) ; 
  globabbr ( 127 ) = globabbr ( 127 ) * globabbr ( 5 ) ; 
  globabbr ( 36 ) = globabbr ( 127 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 36 ) ; 
  globabbr ( 36 ) = globabbr ( 36 ) * globabbr ( 65 ) ; 
  globabbr ( 121 ) = - globabbr ( 121 ) * globabbr ( 133 ) ; 
  globabbr ( 21 ) = globabbr ( 121 ) - globabbr ( 53 ) + globabbr ( 118 ) + globabbr ( 21 ) - globabbr ( 36 ) 
    + globabbr ( 40 ) ; 
  globabbr ( 40 ) = - globabbr ( 157 ) * globabbr ( 89 ) ; 
  globabbr ( 40 ) = globabbr ( 40 ) + globabbr ( 123 ) + NINJA_REAL(4)* globabbr ( 140 ) ; 
  globabbr ( 118 ) = globabbr ( 54 ) * spbl4e2 ; 
  globabbr ( 121 ) = globabbr ( 118 ) * spae2l4 ; 
  globabbr ( 123 ) = globabbr ( 9 ) * globabbr ( 110 ) ; 
  globabbr ( 76 ) = globabbr ( 10 ) * globabbr ( 76 ) ; 
  globabbr ( 133 ) = spak1l5 * globabbr ( 76 ) ; 
  globabbr ( 115 ) = globabbr ( 115 ) + NINJA_REAL(2)* globabbr ( 128 ) + globabbr ( 133 ) + globabbr ( 123 ) - 
    globabbr ( 121 ) ; 
  globabbr ( 123 ) = - NINJA_REAL(16)* globabbr ( 108 ) ; 
  globabbr ( 133 ) = NINJA_REAL(2)* globabbr ( 8 ) ; 
  globabbr ( 140 ) = globabbr ( 132 ) - globabbr ( 56 ) ; 
  globabbr ( 147 ) = NINJA_REAL(2)* globabbr ( 77 ) - globabbr ( 94 ) ; 
  globabbr ( 147 ) = globabbr ( 147 ) * globabbr ( 55 ) ; 
  globabbr ( 154 ) = globabbr ( 4 ) * globabbr ( 11 ) ; 
  globabbr ( 72 ) = NINJA_REAL(2)* globabbr ( 154 ) - NINJA_REAL(3)* globabbr ( 56 ) + globabbr ( 147 ) + globabbr ( 119 ) - 
    globabbr ( 72 ) ; 
  globabbr ( 13 ) = - globabbr ( 36 ) - globabbr ( 25 ) + globabbr ( 13 ) + globabbr ( 53 ) ; 
  globabbr ( 25 ) = - globabbr ( 77 ) + NINJA_REAL(3)/ NINJA_REAL(2)* globabbr ( 94 ) ; 
  globabbr ( 25 ) = globabbr ( 25 ) * globabbr ( 109 ) ; 
  globabbr ( 25 ) = - NINJA_REAL(1)/ NINJA_REAL(4)* globabbr ( 135 ) + globabbr ( 56 ) + globabbr ( 25 ) + globabbr ( 8 ) ; 
  globabbr ( 25 ) = es12 * globabbr ( 25 ) ; 
  globabbr ( 36 ) = globabbr ( 49 ) * globabbr ( 124 ) ; 
  globabbr ( 49 ) = - globabbr ( 62 ) * globabbr ( 64 ) ; 
  globabbr ( 49 ) = globabbr ( 49 ) + NINJA_REAL(1)/ NINJA_REAL(4)* globabbr ( 59 ) ; 
  globabbr ( 49 ) = globabbr ( 139 ) * globabbr ( 49 ) ; 
  globabbr ( 53 ) = - globabbr ( 113 ) * globabbr ( 122 ) ; 
  globabbr ( 13 ) = NINJA_REAL(1)/ NINJA_REAL(4)* globabbr ( 134 ) + globabbr ( 53 ) + globabbr ( 36 ) + globabbr ( 25 ) + 
    globabbr ( 49 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 13 ) ; 
  globabbr ( 25 ) = - globabbr ( 99 ) * globabbr ( 81 ) ; 
  globabbr ( 25 ) = - NINJA_REAL(5)/ NINJA_REAL(2)* globabbr ( 75 ) + globabbr ( 25 ) ; 
  globabbr ( 25 ) = spae1k1 * globabbr ( 25 ) ; 
  globabbr ( 36 ) = - spak1l5 * globabbr ( 126 ) ; 
  globabbr ( 36 ) = - NINJA_REAL(5)/ NINJA_REAL(2)* globabbr ( 138 ) + globabbr ( 36 ) ; 
  globabbr ( 36 ) = spbk1e1 * globabbr ( 36 ) ; 
  globabbr ( 49 ) = globabbr ( 33 ) - globabbr ( 7 ) ; 
  globabbr ( 53 ) = globabbr ( 49 ) * globabbr ( 42 ) ; 
  globabbr ( 59 ) = - globabbr ( 152 ) * globabbr ( 141 ) ; 
  globabbr ( 62 ) = - globabbr ( 54 ) * globabbr ( 157 ) ; 
  globabbr ( 75 ) = globabbr ( 87 ) * globabbr ( 98 ) ; 
  globabbr ( 25 ) = globabbr ( 75 ) + globabbr ( 62 ) - NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 27 ) + globabbr ( 36 ) + 
    globabbr ( 59 ) - NINJA_REAL(3)* globabbr ( 53 ) + globabbr ( 25 ) ; 
  globabbr ( 27 ) = globabbr ( 97 ) * spak1l5 ; 
  globabbr ( 36 ) = - globabbr ( 27 ) + globabbr ( 145 ) ; 
  globabbr ( 36 ) = spbe2k1 * globabbr ( 36 ) ; 
  globabbr ( 53 ) = - globabbr ( 49 ) * globabbr ( 110 ) ; 
  globabbr ( 59 ) = - globabbr ( 33 ) * globabbr ( 112 ) ; 
  globabbr ( 36 ) = globabbr ( 59 ) - globabbr ( 128 ) + globabbr ( 36 ) + globabbr ( 53 ) + NINJA_REAL(1)/ NINJA_REAL(2)* 
    globabbr ( 121 ) - globabbr ( 142 ) ; 
  globabbr ( 53 ) = NINJA_REAL(5)* globabbr ( 108 ) ; 
  globabbr ( 59 ) = globabbr ( 105 ) * globabbr ( 109 ) ; 
  globabbr ( 8 ) = globabbr ( 59 ) - globabbr ( 8 ) ; 
  globabbr ( 62 ) = - NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 140 ) ; 
  globabbr ( 11 ) = - globabbr ( 148 ) - globabbr ( 11 ) ; 
  globabbr ( 11 ) = globabbr ( 4 ) * globabbr ( 11 ) ; 
  globabbr ( 75 ) = - globabbr ( 109 ) * globabbr ( 92 ) ; 
  globabbr ( 11 ) = globabbr ( 75 ) + globabbr ( 11 ) + globabbr ( 149 ) + globabbr ( 136 ) + globabbr ( 8 ) ; 
  globabbr ( 59 ) = globabbr ( 59 ) + globabbr ( 133 ) - globabbr ( 149 ) ; 
  globabbr ( 75 ) = - globabbr ( 149 ) + globabbr ( 8 ) ; 
  globabbr ( 75 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 75 ) ; 
  globabbr ( 81 ) = globabbr ( 37 ) * spbl4k2 ; 
  globabbr ( 46 ) = - globabbr ( 81 ) - globabbr ( 46 ) - NINJA_REAL(3)* globabbr ( 88 ) ; 
  globabbr ( 46 ) = spae2k2 * globabbr ( 46 ) ; 
  globabbr ( 92 ) = - globabbr ( 28 ) * globabbr ( 106 ) ; 
  globabbr ( 46 ) = globabbr ( 46 ) + globabbr ( 92 ) ; 
  globabbr ( 84 ) = spae2k2 * globabbr ( 84 ) ; 
  globabbr ( 45 ) = NINJA_REAL(2)* globabbr ( 84 ) + globabbr ( 45 ) ; 
  globabbr ( 84 ) = globabbr ( 130 ) * spae1k1 ; 
  globabbr ( 92 ) = globabbr ( 93 ) * spae1l4 ; 
  globabbr ( 98 ) = globabbr ( 92 ) * globabbr ( 9 ) ; 
  globabbr ( 12 ) = globabbr ( 12 ) * spbl5e2 ; 
  globabbr ( 84 ) = globabbr ( 12 ) + globabbr ( 84 ) + globabbr ( 98 ) ; 
  globabbr ( 98 ) = es12 * globabbr ( 84 ) ; 
  globabbr ( 105 ) = globabbr ( 23 ) * spae1e2 ; 
  globabbr ( 106 ) = globabbr ( 86 ) * globabbr ( 105 ) ; 
  globabbr ( 98 ) = globabbr ( 106 ) + globabbr ( 98 ) ; 
  globabbr ( 106 ) = NINJA_REAL(7)* globabbr ( 7 ) ; 
  globabbr ( 108 ) = - globabbr ( 106 ) - globabbr ( 9 ) ; 
  globabbr ( 108 ) = globabbr ( 108 ) * spae1k1 ; 
  globabbr ( 110 ) = - spae1l4 * spak1k2 * globabbr ( 125 ) ; 
  globabbr ( 108 ) = globabbr ( 110 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 108 ) ; 
  globabbr ( 108 ) = spbl4k1 * globabbr ( 108 ) ; 
  globabbr ( 110 ) = globabbr ( 16 ) + globabbr ( 85 ) ; 
  globabbr ( 110 ) = globabbr ( 5 ) * globabbr ( 110 ) ; 
  globabbr ( 81 ) = - globabbr ( 81 ) + globabbr ( 110 ) + globabbr ( 88 ) ; 
  globabbr ( 88 ) = - globabbr ( 117 ) - globabbr ( 107 ) ; 
  globabbr ( 86 ) = globabbr ( 86 ) * globabbr ( 88 ) ; 
  globabbr ( 81 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 81 ) + globabbr ( 86 ) ; 
  globabbr ( 81 ) = spae1k2 * globabbr ( 81 ) ; 
  globabbr ( 86 ) = globabbr ( 152 ) * globabbr ( 28 ) * spae1l5 ; 
  globabbr ( 73 ) = - spbl5k1 * globabbr ( 73 ) ; 
  globabbr ( 73 ) = globabbr ( 73 ) + globabbr ( 86 ) + globabbr ( 81 ) + globabbr ( 108 ) ; 
  globabbr ( 81 ) = globabbr ( 84 ) + NINJA_REAL(2)* globabbr ( 105 ) ; 
  globabbr ( 84 ) = globabbr ( 155 ) + globabbr ( 107 ) ; 
  globabbr ( 84 ) = spae1k2 * globabbr ( 84 ) ; 
  globabbr ( 10 ) = globabbr ( 10 ) * spae1k1 ; 
  globabbr ( 10 ) = - globabbr ( 92 ) - globabbr ( 10 ) ; 
  globabbr ( 10 ) = globabbr ( 33 ) * globabbr ( 10 ) ; 
  globabbr ( 10 ) = - NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 12 ) - globabbr ( 105 ) + globabbr ( 10 ) ; 
  globabbr ( 12 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 87 ) ; 
  globabbr ( 33 ) = globabbr ( 103 ) + globabbr ( 89 ) ; 
  globabbr ( 86 ) = spbe2k1 * spak1k2 ; 
  globabbr ( 88 ) = - spbl5e2 * spak2l5 ; 
  globabbr ( 86 ) = globabbr ( 88 ) + globabbr ( 86 ) ; 
  globabbr ( 33 ) = globabbr ( 33 ) * globabbr ( 86 ) ; 
  globabbr ( 86 ) = - spbl4e2 * globabbr ( 127 ) ; 
  globabbr ( 88 ) = - spak2l4 * globabbr ( 118 ) ; 
  globabbr ( 33 ) = globabbr ( 88 ) + globabbr ( 86 ) + globabbr ( 33 ) ; 
  globabbr ( 78 ) = globabbr ( 61 ) + NINJA_REAL(2)* globabbr ( 78 ) ; 
  globabbr ( 86 ) = spbl4e2 * globabbr ( 78 ) ; 
  globabbr ( 86 ) = - globabbr ( 143 ) + globabbr ( 86 ) ; 
  globabbr ( 88 ) = spbl5e1 * spae2l5 ; 
  globabbr ( 24 ) = globabbr ( 88 ) + globabbr ( 24 ) ; 
  globabbr ( 88 ) = globabbr ( 19 ) * globabbr ( 24 ) ; 
  globabbr ( 89 ) = globabbr ( 19 ) * spae2l4 ; 
  globabbr ( 92 ) = globabbr ( 89 ) * spbl4e1 ; 
  globabbr ( 20 ) = globabbr ( 20 ) * spbe2e1 ; 
  globabbr ( 105 ) = globabbr ( 58 ) * globabbr ( 31 ) ; 
  globabbr ( 88 ) = - globabbr ( 105 ) + NINJA_REAL(2)* globabbr ( 20 ) + globabbr ( 44 ) - globabbr ( 88 ) - 
    globabbr ( 92 ) ; 
  globabbr ( 92 ) = - es12 * globabbr ( 88 ) ; 
  globabbr ( 108 ) = globabbr ( 114 ) + globabbr ( 17 ) ; 
  globabbr ( 108 ) = globabbr ( 5 ) * globabbr ( 108 ) ; 
  globabbr ( 54 ) = globabbr ( 54 ) * globabbr ( 101 ) ; 
  globabbr ( 101 ) = globabbr ( 99 ) * spbl4k1 ; 
  globabbr ( 110 ) = - spak1l4 * globabbr ( 101 ) ; 
  globabbr ( 54 ) = globabbr ( 110 ) + globabbr ( 54 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 108 ) ; 
  globabbr ( 54 ) = spbl4e1 * globabbr ( 54 ) ; 
  globabbr ( 103 ) = - globabbr ( 103 ) * globabbr ( 124 ) ; 
  globabbr ( 67 ) = - globabbr ( 99 ) * globabbr ( 67 ) ; 
  globabbr ( 67 ) = globabbr ( 103 ) + globabbr ( 67 ) ; 
  globabbr ( 67 ) = spbl5e1 * globabbr ( 67 ) ; 
  globabbr ( 43 ) = globabbr ( 43 ) - globabbr ( 58 ) ; 
  globabbr ( 43 ) = spbl4k1 * globabbr ( 43 ) ; 
  globabbr ( 43 ) = globabbr ( 43 ) - globabbr ( 80 ) ; 
  globabbr ( 43 ) = globabbr ( 43 ) * globabbr ( 68 ) ; 
  globabbr ( 58 ) = globabbr ( 77 ) + NINJA_REAL(7)* globabbr ( 94 ) ; 
  globabbr ( 58 ) = spbk1e1 * spak1k2 * globabbr ( 58 ) ; 
  globabbr ( 43 ) = globabbr ( 67 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 58 ) + globabbr ( 43 ) + globabbr ( 54 ) ; 
  globabbr ( 54 ) = globabbr ( 99 ) * spbl4e1 ; 
  globabbr ( 58 ) = globabbr ( 14 ) * spbl4e1 ; 
  globabbr ( 63 ) = globabbr ( 74 ) + globabbr ( 63 ) ; 
  globabbr ( 63 ) = spbl4e1 * globabbr ( 63 ) ; 
  globabbr ( 44 ) = globabbr ( 44 ) - globabbr ( 105 ) ; 
  globabbr ( 67 ) = - spae2l4 * spbl4e1 ; 
  globabbr ( 24 ) = globabbr ( 67 ) - globabbr ( 24 ) ; 
  globabbr ( 24 ) = globabbr ( 32 ) * globabbr ( 24 ) ; 
  globabbr ( 20 ) = globabbr ( 20 ) + globabbr ( 24 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 44 ) ; 
  globabbr ( 24 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 58 ) ; 
  globabbr ( 44 ) = - globabbr ( 66 ) * globabbr ( 64 ) ; 
  globabbr ( 67 ) = - globabbr ( 7 ) * globabbr ( 157 ) ; 
  globabbr ( 68 ) = - NINJA_REAL(1)/ NINJA_REAL(2)* spae2l4 * globabbr ( 23 ) ; 
  globabbr ( 74 ) = NINJA_REAL(8)* globabbr ( 7 ) ; 
  globabbr ( 77 ) = globabbr ( 28 ) * globabbr ( 109 ) ; 
  globabbr ( 7 ) = globabbr ( 7 ) * globabbr ( 109 ) ; 
  globabbr ( 47 ) = globabbr ( 47 ) - globabbr ( 150 ) ; 
  globabbr ( 47 ) = globabbr ( 47 ) * globabbr ( 109 ) ; 
  globabbr ( 49 ) = globabbr ( 49 ) * globabbr ( 109 ) ; 
  globabbr ( 64 ) = spbl5l4 * globabbr ( 56 ) * globabbr ( 64 ) ; 
  globabbr ( 60 ) = spae1k2 * globabbr ( 60 ) ; 
  globabbr ( 80 ) = globabbr ( 34 ) * globabbr ( 157 ) ; 
  globabbr ( 94 ) = - spbl5e1 * globabbr ( 95 ) ; 
  globabbr ( 60 ) = globabbr ( 80 ) + globabbr ( 60 ) + globabbr ( 94 ) ; 
  globabbr ( 9 ) = - globabbr ( 51 ) + NINJA_REAL(3)* globabbr ( 9 ) - globabbr ( 106 ) ; 
  globabbr ( 9 ) = globabbr ( 31 ) * globabbr ( 9 ) ; 
  globabbr ( 51 ) = spbl5l4 * globabbr ( 89 ) ; 
  globabbr ( 9 ) = globabbr ( 51 ) + globabbr ( 38 ) + globabbr ( 9 ) ; 
  globabbr ( 38 ) = - spak1k2 * globabbr ( 76 ) ; 
  globabbr ( 51 ) = spak2l5 * globabbr ( 104 ) ; 
  globabbr ( 51 ) = - globabbr ( 111 ) + globabbr ( 51 ) ; 
  globabbr ( 51 ) = spbl5e2 * globabbr ( 51 ) ; 
  globabbr ( 9 ) = globabbr ( 51 ) + NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 9 ) + globabbr ( 38 ) ; 
  globabbr ( 38 ) = - NINJA_REAL(4)* globabbr ( 34 ) ; 
  globabbr ( 34 ) = - globabbr ( 109 ) * globabbr ( 34 ) ; 
  globabbr ( 51 ) = NINJA_REAL(1)/ NINJA_REAL(4)* spbl5l4 ; 
  globabbr ( 51 ) = globabbr ( 139 ) * globabbr ( 51 ) ; 
  globabbr ( 16 ) = - globabbr ( 16 ) + globabbr ( 85 ) ; 
  globabbr ( 16 ) = globabbr ( 16 ) * globabbr ( 35 ) ; 
  globabbr ( 35 ) = globabbr ( 65 ) * globabbr ( 37 ) ; 
  globabbr ( 16 ) = globabbr ( 16 ) + globabbr ( 35 ) ; 
  globabbr ( 16 ) = globabbr ( 66 ) + NINJA_REAL(1)/ NINJA_REAL(4)* globabbr ( 16 ) ; 
  globabbr ( 16 ) = spak1k2 * globabbr ( 16 ) ; 
  globabbr ( 35 ) = NINJA_REAL(1)/ NINJA_REAL(4)* globabbr ( 55 ) ; 
  globabbr ( 28 ) = - spbl5l4 * spak1l5 * globabbr ( 28 ) * globabbr ( 35 ) ; 
  globabbr ( 16 ) = globabbr ( 16 ) + globabbr ( 28 ) ; 
  globabbr ( 28 ) = spak1l4 * globabbr ( 126 ) ; 
  globabbr ( 35 ) = - spae1k1 * globabbr ( 82 ) ; 
  globabbr ( 28 ) = globabbr ( 28 ) + globabbr ( 35 ) ; 
  globabbr ( 28 ) = spbl4e1 * globabbr ( 28 ) ; 
  globabbr ( 35 ) = spbl5e1 * globabbr ( 90 ) ; 
  globabbr ( 28 ) = globabbr ( 35 ) + globabbr ( 28 ) ; 
  globabbr ( 27 ) = - spbl5e2 * globabbr ( 27 ) ; 
  globabbr ( 23 ) = - spak1e2 * globabbr ( 23 ) ; 
  globabbr ( 23 ) = globabbr ( 23 ) + globabbr ( 27 ) ; 
  globabbr ( 23 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 23 ) ; 
  globabbr ( 27 ) = - NINJA_REAL(4)* globabbr ( 144 ) ; 
  globabbr ( 35 ) = - globabbr ( 55 ) * globabbr ( 144 ) ; 
  globabbr ( 37 ) = - spak1k2 * globabbr ( 55 ) * globabbr ( 107 ) ; 
  globabbr ( 55 ) = globabbr ( 109 ) * globabbr ( 144 ) ; 
  globabbr ( 65 ) = spbl4e1 * globabbr ( 95 ) ; 
  globabbr ( 18 ) = globabbr ( 93 ) * globabbr ( 18 ) ; 
  globabbr ( 66 ) = NINJA_REAL(4)* globabbr ( 71 ) ; 
  globabbr ( 71 ) = - spal4l5 * globabbr ( 66 ) ; 
  globabbr ( 66 ) = spak1l5 * globabbr ( 66 ) ; 
  globabbr ( 76 ) = NINJA_REAL(4)* spbk2k1 * globabbr ( 14 ) ; 
  globabbr ( 17 ) = globabbr ( 114 ) - globabbr ( 17 ) ; 
  globabbr ( 17 ) = globabbr ( 17 ) * globabbr ( 83 ) ; 
  globabbr ( 56 ) = spak2l4 * globabbr ( 56 ) ; 
  globabbr ( 17 ) = globabbr ( 17 ) + globabbr ( 56 ) ; 
  globabbr ( 17 ) = spbl4k1 * globabbr ( 17 ) ; 
  globabbr ( 56 ) = globabbr ( 116 ) * globabbr ( 69 ) ; 
  globabbr ( 17 ) = globabbr ( 56 ) + globabbr ( 17 ) ; 
  globabbr ( 17 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 17 ) ; 
  globabbr ( 56 ) = - globabbr ( 99 ) + globabbr ( 78 ) ; 
  globabbr ( 56 ) = globabbr ( 70 ) * spbl4e1 * globabbr ( 56 ) ; 
  globabbr ( 69 ) = globabbr ( 157 ) * globabbr ( 101 ) ; 
  globabbr ( 14 ) = spbl5k1 * globabbr ( 14 ) * globabbr ( 42 ) ; 
  globabbr ( 42 ) = spbk1e1 * globabbr ( 95 ) ; 
  globabbr ( 14 ) = globabbr ( 14 ) + globabbr ( 69 ) + globabbr ( 56 ) + NINJA_REAL(3)* globabbr ( 42 ) ; 
  globabbr ( 42 ) = - spbl5k1 * spae2l5 * globabbr ( 19 ) ; 
  globabbr ( 31 ) = - spbk2k1 * globabbr ( 31 ) * globabbr ( 61 ) ; 
  globabbr ( 29 ) = globabbr ( 42 ) + globabbr ( 31 ) + globabbr ( 29 ) ; 
  globabbr ( 29 ) = NINJA_REAL(1)/ NINJA_REAL(2)* globabbr ( 29 ) ; 
  globabbr ( 31 ) = - NINJA_REAL(4)* globabbr ( 41 ) ; 
  globabbr ( 42 ) = - spbl4k1 * globabbr ( 139 ) ; 
  globabbr ( 56 ) = - spbl4k1 * globabbr ( 151 ) ; 
  globabbr ( 41 ) = globabbr ( 109 ) * globabbr ( 41 ) ;  
}
